//+------------------------------------------------------------------+
//|                                        CMomentumOnRingBuffer.mqh |
//|                               Copyright 2012, Konstantin Gruzdev |
//|                            https://login.mql5.com/ru/users/Lizar |
//|                                             Revision 18 Dec 2012 |
//+------------------------------------------------------------------+
#property copyright   "Copyright 2012, Konstantin Gruzdev"
#property link        "https://login.mql5.com/ru/users/Lizar"

//---   :
#include <IncOnRingBuffer\CArrayRing.mqh>
//+------------------------------------------------------------------+
//| Class CMomentumOnRingBuffer                                      |
//| Appointment:              |
//|                (Momentum)    |
//|                   .                    |
//| Link: http://www.mql5.com/ru/code/1396                           |
//+------------------------------------------------------------------+
class CMomentumOnRingBuffer : public CArrayRing
  {
private:
   CArrayRing        m_value_in;       //   
   string            m_name;           //  
   int               m_period;         //  Momentum
   bool              m_as_series;      // true,     
   int               m_bars_required;  //     
   int               m_begin;          //    
   int               m_start;          //        
   int               m_index;          //   
public:
                     CMomentumOnRingBuffer() {} 
                    ~CMomentumOnRingBuffer() {}
   //---  :
   bool              Init(int period=14, int size_buffer=256, bool as_series=false);             
   //---  :          
   int               MainOnArray(const int      rates_total, 
                                 const int      prev_calculated,
                                 const double&  array[]);
   double            MainOnValue(const int      rates_total,
                                 const int      prev_calculated, 
                                 const int      begin, 
                                 const double   value, 
                                 const int      index);
   //---    private-:
   int               BarsRequired()                { return(m_bars_required); }
   string            Name()                        { return(m_name);          }
   int               Period()                      { return(m_period);        }
   //---      :
   double operator   [](const int index) const     { return(At(index));       }
  };

//+------------------------------------------------------------------+
//|                                                |
//+------------------------------------------------------------------+
bool CMomentumOnRingBuffer :: Init(int period=14, int size_buffer=256, bool as_series=false)
  {
//--- check for input values
   if(period<0)
     {
      m_period=14;
      Print("Input parameter Period has wrong value. Indicator will use value ",m_period);
     }
   else m_period=period;
   //---
   int sizebuffer=period;
   if(size_buffer<sizebuffer)
     {
      printf("Input parameter size_buffer has incorrect value (%d). Indicator will use value %d for calculations.",
             size_buffer,sizebuffer);
     }
   else sizebuffer=size_buffer;
//---      :
   if(!CArrayRing::Init(sizebuffer)) return false;
   if(!m_value_in.Init(m_period+1))  return false;
//---
   m_as_series=as_series;
   m_bars_required=m_period+1;
   m_name="Momentum("+IntegerToString(m_period)+")";
//---
   return true;   
  }

//+------------------------------------------------------------------+
//| Indicator on array                                               |
//+------------------------------------------------------------------+
int CMomentumOnRingBuffer :: MainOnArray(const int      rates_total, 
                                         const int      prev_calculated,
                                         const double&  array[])
  {
//--- save as_series flags:
   bool as_series = ArrayGetAsSeries(array);
   if(as_series) ArraySetAsSeries(array, false);
//--- first calculation:
   if(prev_calculated==0)
     {
      //---    :
      int i=-1;
      while(++i<rates_total && !IsStopped())
         if(array[i]!=0 && array[i]!=EMPTY_VALUE) break;
      //---      :
      m_start=MathMax(i,rates_total-Size()-m_bars_required)+m_period;
     }
//--- number of bars was changed:
   else m_start=prev_calculated-1;
   if(array[m_start-m_period]!=0) Last(array[m_start]*100.0/array[m_start-m_period]);
   else Last(EMPTY_VALUE);
//--- main loop:
   for(int i=m_start+1, j=m_start+1-m_period; i<rates_total && !IsStopped(); i++, j++)
     {
      if(array[j]!=0) Add(array[i]*100.0/array[j]);
      else Add(EMPTY_VALUE);
     }
//--- restore as_series flags:
   if(as_series) ArraySetAsSeries(array, true);
//--- return value of prev_calculated for next call:
   return(rates_total);
  }
  
//+------------------------------------------------------------------+
//| Indicator on value                                               |
//+------------------------------------------------------------------+
double CMomentumOnRingBuffer:: MainOnValue(const int    rates_total,
                                           const int    prev_calculated, 
                                           const int    begin, 
                                           const double value, 
                                           const int    index)
  {
//--- check as_series flags:
   if(m_as_series) m_index=rates_total-1-index;
   else m_index=index;
//--- check begin:
   if(m_index<begin) return(EMPTY_VALUE);
//--- fill the ring buffer of input data:
   if(prev_calculated-1==m_index) m_value_in.Last(value);
   else m_value_in.Add(value);
//--- initial calculation:
   if(m_index<begin+m_period) return (EMPTY_VALUE);
//--- main calculation:
   if(prev_calculated-1==m_index)
     {
      if(m_value_in[m_period]!=0) 
         Last(value*100.0/m_value_in[m_period]);
      else Last(EMPTY_VALUE);
     }
   else
     {
      if(m_value_in[m_period]!=0) 
         Add(value*100.0/m_value_in[m_period]);
      else Add(EMPTY_VALUE);
     }
//--- result:
   return(Last());          
  }       
